@@ -22,6 +22,7 @@ import {BrowserWindow, Event, IpcMessageEvent, Menu, app, ipcMain, shell} from '
 import WindowStateKeeper = require('electron-window-state');
 import fileUrl = require('file-url');
 import * as fs from 'fs-extra';
+import {getProxySettings} from 'get-proxy-settings';
 import * as logdown from 'logdown';
 import * as minimist from 'minimist';
 import * as path from 'path';
@@ -39,6 +40,7 @@ import {WebViewFocus} from './lib/webViewFocus';
 import * as locale from './locale/locale';
 import {ENABLE_LOGGING, getLogger} from './logging/getLogger';
 import {Raygun} from './logging/initRaygun';
+import {getLogFiles} from './logging/loggerUtils';
 import {menuItem as developerMenu} from './menu/developer';
 import * as systemMenu from './menu/system';
 import {TrayHandler} from './menu/TrayHandler';
@@ -50,6 +52,7 @@ import {settings} from './settings/ConfigurationPersistence';
 import {SettingsType} from './settings/SettingsType';
 import {SingleSignOn} from './sso/SingleSignOn';
 import {AboutWindow} from './window/AboutWindow';
+import {ProxyPromptWindow} from './window/ProxyPromptWindow';
 import {WindowManager} from './window/WindowManager';
 import {WindowUtil} from './window/WindowUtil';
 
@@ -67,18 +70,32 @@ const WINDOW_SIZE = {
   MIN_WIDTH: 760,
 };
 
+let authenticatedProxyInfo: URL | undefined;
+
 const customProtocolHandler = new CustomProtocolHandler();
 
 // Config
 const argv = minimist(process.argv.slice(1));
 const BASE_URL = EnvironmentUtil.web.getWebappUrl(argv.env);
 
+const logger = getLogger(__filename);
+
 if (argv.version) {
   console.log(config.version);
   process.exit();
 }
 
-const logger = getLogger(__filename);
+if (argv['proxy-server-auth']) {
+  try {
+    authenticatedProxyInfo = new URL(argv['proxy-server-auth']);
+    if (authenticatedProxyInfo.origin === 'null') {
+      authenticatedProxyInfo = undefined;
+      throw new Error('No protocol for the proxy server specified.');
+    }
+  } catch (error) {
+    logger.error(`Could not parse authenticated proxy URL: "${error.message}"`);
+  }
+}
 
 // Icon
 const ICON = `wire.${EnvironmentUtil.platform.IS_WINDOWS ? 'ico' : 'png'}`;
@@ -87,6 +104,7 @@ let tray: TrayHandler;
 
 let isFullScreen = false;
 let isQuitting = false;
+let triedProxy = false;
 let main: BrowserWindow;
 
 Object.entries(config).forEach(([key, value]) => {
@@ -261,14 +279,24 @@ const showMainWindow = (mainWindowState: WindowStateKeeper.State) => {
     systemMenu.unregisterShortcuts();
   });
 
-  main.webContents.on('crashed', () => main.reload());
+  main.webContents.on('crashed', event => {
+    logger.error('WebContents crashed. Will reload the window.');
+    logger.error(event);
+    main.reload();
+  });
+
+  app.on('gpu-process-crashed', event => {
+    logger.error('GPU process crashed. Will reload the window.');
+    logger.error(event);
+    main.reload();
+  });
 };
 
 // App Events
 const handleAppEvents = () => {
-  app.on('window-all-closed', async () => {
+  app.on('window-all-closed', () => {
     if (!EnvironmentUtil.platform.IS_MAC_OS) {
-      await lifecycle.quit();
+      lifecycle.quit();
     }
   });
 
@@ -283,6 +311,63 @@ const handleAppEvents = () => {
     isQuitting = true;
   });
 
+  app.on('login', async (event, webContents, request, authInfo, callback) => {
+    if (authInfo.isProxy) {
+      event.preventDefault();
+
+      if (authenticatedProxyInfo) {
+        const {username, password} = authenticatedProxyInfo;
+        logger.info('Sending provided credentials to authenticated proxy ...');
+        return callback(username, password);
+      }
+
+      const systemProxy = await getProxySettings();
+      const systemProxySettings = systemProxy && (systemProxy.http || systemProxy.https);
+
+      if (systemProxySettings) {
+        const {
+          credentials: {username, password},
+          protocol,
+        } = systemProxySettings;
+        authenticatedProxyInfo = new URL(`${protocol}//${username}:${password}@${authInfo.host}`);
+        return callback(username, password);
+      }
+
+      ipcMain.once(
+        EVENT_TYPE.PROXY_PROMPT.SUBMITTED,
+        (event: IpcMessageEvent, promptData: {password: string; username: string}) => {
+          const {username, password} = promptData;
+
+          logger.log('Proxy prompt was submitted');
+          const [originalProxyValue] = argv['proxy-server'] || argv['proxy-server-auth'] || ['http://'];
+          const protocol = /^[^:]+:\/\//.exec(originalProxyValue);
+          authenticatedProxyInfo = new URL(`${protocol}${username}:${password}@${authInfo.host}`);
+          callback(username, password);
+        },
+      );
+
+      ipcMain.once(EVENT_TYPE.PROXY_PROMPT.CANCELED, async () => {
+        logger.log('Proxy prompt was canceled');
+        webContents.session.setProxy(
+          {
+            pacScript: '',
+            proxyBypassRules: '',
+            proxyRules: '',
+          },
+          () => {
+            callback('', '');
+            main.reload();
+          },
+        );
+      });
+
+      if (!triedProxy) {
+        ProxyPromptWindow.showWindow();
+        triedProxy = true;
+      }
+    }
+  });
+
   // System Menu, Tray Icon & Show window
   app.on('ready', () => {
     const mainWindowState = initWindowStateKeeper();
@@ -301,40 +386,26 @@ const handleAppEvents = () => {
 };
 
 const renameFileExtensions = (files: string[], oldExtension: string, newExtension: string): void => {
-  files
-    .filter(file => {
-      try {
-        return fs.statSync(file).isFile();
-      } catch (statError) {
-        return false;
+  for (const file of files) {
+    try {
+      const fileStat = fs.statSync(file);
+      if (fileStat.isFile() && file.endsWith(oldExtension)) {
+        fs.renameSync(file, file.replace(oldExtension, newExtension));
       }
-    })
-    .forEach(file => {
-      if (file.endsWith(oldExtension)) {
-        try {
-          fs.renameSync(file, file.replace(oldExtension, newExtension));
-        } catch (error) {
-          logger.error(`Failed to rename log file: "${error.message}"`);
-        }
-      }
-    });
+    } catch (error) {
+      logger.error(`Failed to rename log file: "${error.message}"`);
+    }
+  }
 };
 
 const renameWebViewLogFiles = (): void => {
   // Rename "console.log" to "console.old" (for every log directory of every account)
-  fs.readdir(LOG_DIR, (readError, contents) => {
-    if (readError) {
-      return logger.log(`Failed to read log directory with error: ${readError.message}`);
-    }
-
-    const logFiles = contents.map(file => path.join(LOG_DIR, file, config.logFileName));
+  try {
+    const logFiles = getLogFiles(LOG_DIR, true);
     renameFileExtensions(logFiles, '.log', '.old');
-  });
-};
-
-const initElectronLogFile = (): void => {
-  renameFileExtensions([LOG_FILE], '.log', '.old');
-  fs.ensureFileSync(LOG_FILE);
+  } catch (error) {
+    logger.log(`Failed to read log directory with error: ${error.message}`);
+  }
 };
 
 const addLinuxWorkarounds = () => {
@@ -353,11 +424,12 @@ const addLinuxWorkarounds = () => {
 };
 
 const handlePortableFlags = () => {
-  if (argv.portable || argv.user_data_dir) {
-    const USER_PATH =
-      path.resolve(argv.user_data_dir) || path.join(process.env.APPIMAGE || process.execPath, '../Data');
+  if (argv.user_data_dir || argv.portable) {
+    const USER_PATH = argv.user_data_dir
+      ? path.resolve(argv.user_data_dir)
+      : path.join(process.env.APPIMAGE || process.execPath, '../Data');
 
-    console.log(`Saving user data to ${USER_PATH}`);
+    logger.log(`Saving user data to "${USER_PATH}".`);
     app.setPath('userData', USER_PATH);
   }
 };
@@ -413,9 +485,27 @@ class ElectronWrapperInit {
       }
     };
 
-    app.on('web-contents-created', (webviewEvent: Electron.Event, contents: Electron.WebContents) => {
+    app.on('web-contents-created', async (webviewEvent: Electron.Event, contents: Electron.WebContents) => {
       WebViewFocus.bindTracker(webviewEvent, contents);
 
+      if (authenticatedProxyInfo && authenticatedProxyInfo.origin && contents.session) {
+        const proxyURL = `${authenticatedProxyInfo.protocol}//${authenticatedProxyInfo.origin}`;
+        logger.info(`Setting proxy to URL "${proxyURL}" ...`);
+
+        contents.session.allowNTLMCredentialsForDomains(authenticatedProxyInfo.hostname);
+
+        await new Promise(resolve =>
+          contents.session.setProxy(
+            {
+              pacScript: '',
+              proxyBypassRules: '',
+              proxyRules: proxyURL,
+            },
+            () => resolve(),
+          ),
+        );
+      }
+
       switch (contents.getType()) {
         case 'window': {
           contents.on('will-attach-webview', (event, webPreferences, params) => {
@@ -510,6 +600,6 @@ if (lifecycle.isFirstInstance) {
   bindIpcEvents();
   handleAppEvents();
   renameWebViewLogFiles();
-  initElectronLogFile();
-  new ElectronWrapperInit().run().catch(logger.error);
+  fs.ensureFileSync(LOG_FILE);
+  new ElectronWrapperInit().run().catch(error => logger.error(error));
 }
